
# q-expansion of newform 966.2.i.h, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 966
weight = 2
poly_data = [4, 0, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 0, 0, 1], 2]]

hecke_ring_character_values = [[323, [1, 0, 0, 0]], [829, [0, 0, 1, 0]], [925, [1, 0, 0, 0]]]
aps_data = [[0, 0, 1, 0], [-1, 0, -1, 0], [0, 0, -1, 0], [-1, 1, -1, 2], [3, 0, 3, 0], [0, 0, 0, 1], [0, -1, 0, 0], [0, 1, -2, 1], [0, 0, -1, 0], [-5, 0, 0, 3], [1, 4, 1, 0], [0, 5, -2, 5], [6, 0, 0, -3], [-2, 0, 0, -4], [0, 8, 2, 8], [5, 2, 5, 0], [5, 3, 5, 0], [0, 0, -6, 0], [10, 4, 10, 0], [-4, 0, 0, 7], [0, 4, 0, 0], [0, 1, 1, 1], [1, 0, 0, 8], [0, -9, 2, -9], [-5, 0, 0, 1], [-4, -6, -4, 0], [0, 4, -8, 4], [0, 6, 1, 6], [6, 8, 6, 0], [-16, 0, 0, -1], [-3, 0, 0, -2], [0, 3, 3, 3], [2, -12, 2, 0], [6, 0, 0, -8], [0, -4, 8, -4], [7, 4, 7, 0], [-6, -7, -6, 0], [0, 1, 0, 1], [10, 0, 0, 6], [0, -2, 2, -2], [-16, 0, -16, 0], [-4, 0, 0, -6], [0, -1, 0, -1], [-3, -16, -3, 0], [-6, 0, 0, 2], [6, 8, 6, 0], [2, 0, 0, 2], [-13, 0, 0, 2], [5, -2, 5, 0], [0, -9, 10, -9], [0, -15, -6, -15], [-12, 0, 0, -8], [17, 5, 17, 0], [23, 0, 0, 2], [0, 1, 0, 1], [6, 12, 6, 0], [-3, -1, -3, 0], [0, 0, -7, 0], [0, -2, 0, 0], [-14, 0, 0, -9], [10, -10, 10, 0], [1, 0, 0, -16], [16, 0, 0, 7], [-2, 1, -2, 0], [0, 3, 21, 3], [0, 5, 17, 5], [0, -7, 16, -7], [-21, 0, 0, 5], [-8, -16, -8, 0], [2, 0, 0, 17], [10, -9, 10, 0], [0, 0, 2, 0], [5, -21, 5, 0], [0, -4, -20, -4], [12, 0, 0, 12], [0, -9, -14, -9], [4, 2, 4, 0], [0, 11, 16, 11], [0, 6, 8, 6], [19, 4, 19, 0], [-6, 0, 0, 0], [6, 0, 0, 18], [-12, -4, -12, 0], [30, 0, 0, 0], [0, -10, -3, -10], [0, 9, 23, 9], [-6, 0, 0, 3], [0, -15, -1, -15], [14, 0, 0, 12], [6, 0, 0, -14], [0, -2, 26, -2], [2, 23, 2, 0], [21, -6, 21, 0], [-19, 0, 0, 7], [0, 14, 6, 14], [-4, 0, 0, -2], [0, -9, 29, -9], [-26, 0, -26, 0], [0, 20, 12, 20], [0, 21, -10, 21], [20, 0, 0, 3], [29, -8, 29, 0], [-1, -10, -1, 0], [0, -10, -20, -10], [-8, 23, -8, 0], [-15, -16, -15, 0], [-1, 0, 0, -1], [0, -12, -8, -12], [8, 5, 8, 0], [-7, 0, 0, -24], [0, -20, -9, -20], [16, 0, 16, 0], [2, 0, 0, -24], [0, 7, 0, 0], [1, 0, 0, 1], [-10, 21, -10, 0], [-12, 0, 0, 1], [-2, -2, -2, 0], [0, -9, 33, -9], [-10, 0, 0, -24], [6, 12, 6, 0], [27, 0, 0, -12], [0, 4, -37, 4], [-17, -17, -17, 0], [0, -1, -48, -1], [9, 0, 0, -6], [0, -1, -8, -1], [0, 10, 14, 10], [-21, 0, 0, 11], [0, -11, 0, -11], [-18, 2, -18, 0], [-2, 0, 0, -3], [0, -23, 3, -23], [32, 0, 0, 5], [0, 1, 6, 1], [-1, 0, 0, -29], [-20, 2, -20, 0], [2, 18, 2, 0], [13, 0, 0, -12], [-18, 23, -18, 0], [-14, 0, 0, -20], [0, 3, 19, 3], [-24, 16, -24, 0], [1, 0, 0, 14], [36, 6, 36, 0], [12, 0, 0, -21], [-14, 0, 0, -10], [0, 19, 0, 0], [0, -1, 18, -1], [0, -19, 12, -19], [0, -7, 28, -7], [-6, 0, 0, 10], [24, 0, 0, 21], [0, 4, -40, 4], [18, -5, 18, 0], [-10, 0, 0, 4], [0, -20, 8, -20], [0, -9, -12, -9], [9, 0, 0, 27], [-27, 6, -27, 0], [0, 20, 22, 20], [-18, 0, 0, -14], [-19, 0, 0, -12], [0, -12, -13, -12], [-18, 12, -18, 0], [-8, -1, -8, 0], [-1, -26, -1, 0], [18, -19, 18, 0]]
